@@ -77,6 +77,8 @@
 #include "all-io.h"
 #include "closestream.h"
 #include "ismounted.h"
+
+#define XALLOC_EXIT_CODE MKFS_EX_ERROR
 #include "xalloc.h"
 
 #define MINIX_ROOT_INO 1
@@ -129,8 +131,9 @@ static char *zone_map;
 #define mark_zone(x) (setbit(zone_map,(x)-get_first_zone()+1))
 #define unmark_zone(x) (clrbit(zone_map,(x)-get_first_zone()+1))
 
-static void __attribute__((__noreturn__)) usage(FILE *out)
+static void __attribute__((__noreturn__)) usage(void)
 {
+	FILE *out = stdout;
 	fputs(USAGE_HEADER, out);
 	fprintf(out, _(" %s [options] /dev/name [blocks]\n"), program_invocation_short_name);
 	fputs(USAGE_OPTIONS, out);
@@ -142,10 +145,9 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 	fputs(_(" -c, --check             check the device for bad blocks\n"), out);
 	fputs(_(" -l, --badblocks <file>  list of bad blocks from file\n"), out);
 	fputs(USAGE_SEPARATOR, out);
-	fputs(USAGE_HELP, out);
-	fputs(USAGE_VERSION, out);
-	fprintf(out, USAGE_MAN_TAIL("mkfs.minix(8)"));
-	exit(out == stderr ? MKFS_EX_USAGE : MKFS_EX_OK);
+	printf(USAGE_HELP_OPTIONS(25));
+	printf(USAGE_MAN_TAIL("mkfs.minix(8)"));
+	exit(MKFS_EX_OK);
 }
 
 #ifdef TEST_SCRIPT
@@ -758,6 +760,8 @@ int main(int argc, char ** argv)
 	textdomain(PACKAGE);
 	atexit(close_stdout);
 
+	strutils_set_exitcode(MKFS_EX_USAGE);
+
 	while ((i = getopt_long(argc, argv, "1v23n:i:cl:Vh", longopts, NULL)) != -1)
 		switch (i) {
 		case '1':
@@ -791,9 +795,9 @@ int main(int argc, char ** argv)
 			printf(UTIL_LINUX_VERSION);
 			return MKFS_EX_OK;
 		case 'h':
-			usage(stdout);
+			usage();
 		default:
-			usage(stderr);
+			errtryhelp(MKFS_EX_USAGE);
 		}
 	argc -= optind;
 	argv += optind;
@@ -806,7 +810,8 @@ int main(int argc, char ** argv)
 		ctl.fs_blocks = strtoul_or_err(argv[0], _("failed to parse number of blocks"));
 
 	if (!ctl.device_name) {
-		usage(stderr);
+		warnx(_("no device specified"));
+		errtryhelp(MKFS_EX_USAGE);
 	}
 	check_user_instructions(&ctl);
 	if (is_mounted(ctl.device_name))
